@@ -1,4 +1,4 @@
-# Copyright 2001-9 by Roger Bivand 
+# Copyright 2001-9, 2021 by Roger Bivand 
 #
 
 
@@ -7,10 +7,12 @@ geary <- function(x, listw, n, n1, S0, zero.policy=NULL) {
             zero.policy <- get("zeroPolicy", envir = .spdepOptions)
         stopifnot(is.logical(zero.policy))
         stopifnot(is.vector(x))
-	z <- scale(x, scale=FALSE)
+#	z <- scale(x, scale=FALSE)
+        z <- scale(x)
 	zz <- sum(z^2)
 	K <- (n*sum(z^4))/(zz^2)
-	res <- geary.intern(x, listw, n, zero.policy, type="geary")
+#	res <- geary.intern(x, listw, n, zero.policy, type="geary")
+	res <- geary.intern(z, listw, n, zero.policy, type="geary")
 	C <- (n1 / (2*S0)) * (sum(res) / zz)
 	res <- list(C=C, K=K)
 	res
@@ -27,7 +29,7 @@ geary.intern <- function(x, listw, n, zero.policy=NULL, type="geary") {
 	res <- .Call("gearyw", listw$neighbours, listw$weights,
 		as.numeric(x), as.integer(cardnb),
 		as.logical(zero.policy), as.logical(ft), PACKAGE="spdep")
-	if (any(is.na(res))) warning("NAs in lagged values")
+#	if (any(is.na(res))) warning("NAs in lagged values")
 	res
 }
 
@@ -99,7 +101,7 @@ geary.mc <- function(x, listw, nsim, zero.policy=NULL,
             zero.policy <- get("zeroPolicy", envir = .spdepOptions)
         stopifnot(is.logical(zero.policy))
         stopifnot(is.vector(x))
-	alternative <- match.arg(alternative, c("less", "greater"))
+	alternative <- match.arg(alternative, c("less", "greater", "two.sided"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if(!is.numeric(x)) stop(paste(deparse(substitute(x)),
@@ -120,21 +122,10 @@ geary.mc <- function(x, listw, nsim, zero.policy=NULL,
                 var <- var[i]
                 return(geary(x=var, ...)$C)
             }
-            cores <- get.coresOption()
-            if (is.null(cores)) {
-            parallel <- "no"
-            } else {
-                parallel <- ifelse (get.mcOption(), "multicore", "snow")
-            }
-            ncpus <- ifelse(is.null(cores), 1L, cores)
-            cl <- NULL
-            if (parallel == "snow") {
-                cl <- get.ClusterOption()
-                if (is.null(cl)) {
-                    parallel <- "no"
-                    warning("no cluster in ClusterOption, parallel set to no")
-                }
-            }
+            p_setup <- parallel_setup(NULL)
+            parallel <- p_setup$parallel
+            ncpus <- p_setup$ncpus
+            cl <- p_setup$cl
             res <- boot(x, statistic=geary_boot, R=nsim,
                 sim="permutation", listw=listw, n=n, n1=wc$n1, S0=wc$S0, 
                 zero.policy=zero.policy, parallel=parallel, ncpus=ncpus, cl=cl)
@@ -153,6 +144,8 @@ geary.mc <- function(x, listw, nsim, zero.policy=NULL,
         	pval <- punif((diff + 1)/(nsim + 1), lower.tail=FALSE)
     	else if (alternative == "less") 
         	pval <- punif((diff + 1)/(nsim + 1))
+        else pval <- punif(abs(xrank - (nsim+1)/2)/(nsim + 1), 0, 0.5,
+                lower.tail=FALSE)
 	if (!is.finite(pval) || pval < 0 || pval > 1) 
 		warning("Out-of-range p-value: reconsider test arguments")
 	statistic <- res[nsim+1]
