@@ -1,6 +1,93 @@
+parallel_setup <- function(iseed) {
+    cores <- get.coresOption()
+    if (is.null(cores)) {
+        parallel <- "no"
+    } else {
+        parallel <- ifelse (get.mcOption(), "multicore", "snow")
+    }
+    ncpus <- ifelse(is.null(cores), 1L, cores)
+    cl <- NULL
+    if (parallel == "snow") {
+        cl <- get.ClusterOption()
+        if (is.null(cl)) {
+            parallel <- "no"
+            warning("no cluster in ClusterOption, parallel set to no")
+        }
+    }
+    if (!is.null(iseed)) {
+        stopifnot(is.numeric(iseed))
+        stopifnot(length(iseed) == 1L)
+    }
+    list(parallel=parallel, ncpus=ncpus, cl=cl)
+}
+
+spdep_splitIndices <- function(idx, lcl) {
+    n <- length(idx)
+    stopifnot(n == length(unique(idx)))
+    splits <- parallel::splitIndices(n, lcl)
+    res <- lapply(splits, function(i) idx[i])
+    res
+}
+
+run_perm <- function(fun, idx, env, iseed, varlist) {
+    p_setup <- parallel_setup(iseed)
+    parallel <- p_setup$parallel
+    ncpus <- p_setup$ncpus
+    cl <- p_setup$cl
+    if (parallel == "snow") {
+      if (requireNamespace("parallel", quietly = TRUE)) {
+        sI <- spdep_splitIndices(idx, length(cl))
+        parallel::clusterExport(cl, varlist=varlist, envir=env)
+        if (!is.null(iseed)) parallel::clusterSetRNGStream(cl, iseed = iseed)
+        oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
+ 	    fun(i, env)})
+        out <- do.call("rbind", do.call("c", oo))
+      } else {
+        stop("parallel not available")
+      }
+    } else if (parallel == "multicore") {
+      if (requireNamespace("parallel", quietly = TRUE)) {
+        sI <- spdep_splitIndices(idx, ncpus)
+        oldRNG <- RNGkind()
+        RNGkind("L'Ecuyer-CMRG")
+        if (!is.null(iseed)) set.seed(iseed)
+        oo <- parallel::mclapply(sI, FUN=lapply, function(i) {fun(i,
+            env)}, mc.cores=ncpus)
+        RNGkind(oldRNG[1])
+        out <- do.call("rbind", do.call("c", oo))
+      } else {
+        stop("parallel not available")
+      }
+    } else {
+        if (!is.null(iseed)) set.seed(iseed)
+        oo <- lapply(idx, function(i) fun(i, env))
+        out <- do.call("rbind", oo)
+    }
+    out
+}
+
+probs_lut <- function(stat="I", nsim, alternative) {
+    gr <- punif((1:(nsim+1))/(nsim+1), 0, 1)
+    ls <- rev(gr)
+    ts <- (ifelse(gr > ls, ls, gr))*2
+    if (alternative == "two.sided") {
+        probs <- ts
+        Prname <- paste0("Pr(z != E(", stat, "i))")
+    } else if (alternative == "greater") {
+        Prname <- paste0("Pr(z > E(", stat, "i))")
+        probs <- gr
+    } else {
+        Prname <- paste0("Pr(z < E(", stat, "i))")
+        probs <- ls
+    }
+    attr(probs, "Prname") <- Prname
+    probs
+}
+
 localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     na.action=na.fail, alternative = "two.sided",
-    mlvar=TRUE, spChk=NULL, adjust.x=FALSE, sample_Ei=TRUE, iseed=NULL) {
+    mlvar=TRUE, spChk=NULL, adjust.x=FALSE, sample_Ei=TRUE, iseed=NULL,
+    no_repeat_in_row=FALSE) {
     alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
     stopifnot(is.vector(x))
     if (!inherits(listw, "listw"))
@@ -26,25 +113,8 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
         excl <- class(na.act) == "exclude"
     }
     n <- length(listw$neighbours)
-    if (n != length(x))stop("Different numbers of observations")
+    if (n != length(x)) stop("Different numbers of observations")
     res <- matrix(nrow=n, ncol=9)
-    gr <- punif((1:(nsim+1))/(nsim+1), 0, 1)
-    ls <- rev(gr)
-    ts <- (ifelse(gr > ls, ls, gr))*2
-    if (alternative == "two.sided") {
-        probs <- ts
-        Prname <- "Pr(z != E(Ii))"
-    } else if (alternative == "greater") {
-        Prname <- "Pr(z > E(Ii))"
-        probs <- gr
-    } else {
-        Prname <- "Pr(z < E(Ii))"
-        probs <- ls
-    }
-    Prname_rank <- paste0(Prname, " Sim")
-    Prname_sim <- "Pr(folded) Sim"
-    colnames(res) <- c("Ii", "E.Ii", "Var.Ii", "Z.Ii", Prname, Prname_rank, 
-        Prname_sim, "Skewness", "Kurtosis")
     if (adjust.x) {
         nc <- card(listw$neighbours) > 0L
 	xx <- mean(x[nc], na.rm=NAOK)
@@ -82,88 +152,59 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     }
     res[,1] <- (z/s2) * lz
 
-    cores <- get.coresOption()
-    if (is.null(cores)) {
-        parallel <- "no"
-    } else {
-        parallel <- ifelse (get.mcOption(), "multicore", "snow")
-    }
-    ncpus <- ifelse(is.null(cores), 1L, cores)
-    cl <- NULL
-    if (parallel == "snow") {
-        cl <- get.ClusterOption()
-        if (is.null(cl)) {
-            parallel <- "no"
-            warning("no cluster in ClusterOption, parallel set to no")
-        }
-    }
-    if (!is.null(iseed)) {
-        stopifnot(is.numeric(iseed))
-        stopifnot(length(iseed) == 1L)
-    }
-
     crd <- card(listw$neighbours)
-    permI_int <- function(i, zi, z_i, crdi, wtsi, nsim, Ii) {
-        res_i <- rep(as.numeric(NA), 8)       
-        if (crdi > 0) {
-            sz_i <- matrix(sample(z_i, size=crdi*nsim, replace=TRUE),
-                ncol=crdi, nrow=nsim)
-            lz_i <- sz_i %*% wtsi
-            res_p <- (zi/s2)*lz_i
+    lww <- listw$weights
+    Iis <- res[,1]
+
+    env <- new.env()
+    assign("z", z, envir=env)
+    assign("crd", crd, envir=env)
+    assign("lww", lww, envir=env)
+    assign("nsim", nsim, envir=env)
+    assign("Iis", Iis, envir=env)
+    assign("s2", s2, envir=env)
+    assign("n", n, envir=env)
+    assign("no_repeat_in_row", no_repeat_in_row, envir=env)
+    varlist <- ls(envir = env)
+
+    permI_int <- function(i, env) {
+        res_i <- rep(as.numeric(NA), 6) # initialize output
+        crdi <- get("crd", envir=env)[i]
+        no_repeat_in_row <- get("no_repeat_in_row", envir=env)
+        if (crdi > 0) { # if i has neighbours
+            nsim <- get("nsim", envir=env)
+            n_i <- get("n", envir=env) - 1L
+            zi <- get("z", envir=env)[i]
+            z_i <- get("z", envir=env)[-i]
+            if (no_repeat_in_row) {
+              samples <- .Call("perm_no_replace", as.integer(nsim),
+                as.integer(n_i), as.integer(crdi), PACKAGE="spdep")
+              sz_i <- matrix(z_i[samples], ncol=crdi, nrow=nsim)
+            } else {
+              sz_i <- matrix(sample(z_i, size=crdi*nsim, replace=TRUE),
+                ncol=crdi, nrow=nsim) # permute nsim*#neighbours from z[-i]
+            }
+            wtsi <- get("lww", envir=env)[[i]]
+            lz_i <- sz_i %*% wtsi # nsim by 1 = nsim by crdi %*% crdi by 1
+            # create nsim samples of Ii at i
+            s2 <- get("s2", envir=env)
+            res_p <- (zi/s2)*lz_i # nsim by 1 = scalar/scalar * nsim by 1
             res_i[1] <- mean(res_p)
             res_i[2] <- var(res_p)
+            Ii <- get("Iis", envir=env)[i]
             xrank <- rank(c(res_p, Ii))[(nsim + 1L)]
-	    res_i[5] <- xrank
+	    res_i[3] <- xrank
             rnk0 <- as.integer(sum(res_p >= Ii))
             drnk0 <- nsim - rnk0
             rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
-            res_i[6] <- rnk0
-            res_i[7] <- e1071::skewness(res_p)
-            res_i[8] <- e1071::kurtosis(res_p)
+            res_i[4] <- rnk0
+            res_i[5] <- e1071::skewness(res_p)
+            res_i[6] <- e1071::kurtosis(res_p)
         }
         res_i
     }
 
-    lww <- listw$weights
-    Iis <- res[,1]
-    if (parallel == "snow") {
-      if (requireNamespace("parallel", quietly = TRUE)) {
-        sI <- parallel::splitIndices(n, length(cl))
-        env <- new.env()
-        assign("z", z, envir=env)
-        assign("crd", crd, envir=env)
-        assign("lww", lww, envir=env)
-        assign("nsim", nsim, envir=env)
-        assign("Iis", Iis, envir=env)
-        parallel::clusterExport(cl, varlist=c("z", "crd", "lww", "nsim",
-            "Iis"), envir=env)
-        if (!is.null(iseed)) parallel::clusterSetRNGStream(cl, iseed = iseed)
-        oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
- 	    permI_int(i, z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i])})
-        out <- do.call("rbind", do.call("c", oo))
-        rm(env)
-      } else {
-        stop("parallel not available")
-      }
-    } else if (parallel == "multicore") {
-      if (requireNamespace("parallel", quietly = TRUE)) {
-        sI <- parallel::splitIndices(n, ncpus)
-        oldRNG <- RNGkind()
-        RNGkind("L'Ecuyer-CMRG")
-        if (!is.null(iseed)) set.seed(iseed)
-        oo <- parallel::mclapply(sI, FUN=lapply, function(i) {permI_int(i,
-            z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i])}, mc.cores=ncpus)
-        RNGkind(oldRNG[1])
-        out <- do.call("rbind", do.call("c", oo))
-      } else {
-        stop("parallel not available")
-      }
-    } else {
-        if (!is.null(iseed)) set.seed(iseed)
-        oo <- lapply(1:n, function(i) permI_int(i, z[i], z[-i], 
-            crd[i], lww[[i]], nsim, Iis[i]))
-        out <- do.call("rbind", oo)
-    }
+    out <- run_perm(fun=permI_int, idx=1:n, env=env, iseed=iseed, varlist=varlist)
 
     if (sample_Ei) res[,2] <- out[,1]
     else  res[,2] <- EIc
@@ -174,14 +215,24 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     else if (alternative == "greater") 
         res[,5] <- pnorm(res[,4], lower.tail=FALSE)
     else res[,5] <- pnorm(res[,4])
-    res[,6] <- probs[as.integer(out[,5])]
+# look-up table
+    probs <- probs_lut(stat="I", nsim=nsim, alternative=alternative)
+    Prname <- attr(probs, "Prname")
+    Prname_rank <- paste0(Prname, " Sim")
+    Prname_sim <- "Pr(folded) Sim"
+    res[,6] <- probs[as.integer(out[,3])]
 # 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
-    rnk0 <- as.integer(out[,6])
+    rnk0 <- as.integer(out[,4])
     drnk0 <- nsim - rnk0
     rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
+# folded
     res[,7] <- (rnk + 1.0) / (nsim + 1.0)
-    res[,8] <- out[,7]
-    res[,9] <- out[,8]
+# skewness
+    res[,8] <- out[,5]
+# kurtosis
+    res[,9] <- out[,6]
+    colnames(res) <- c("Ii", "E.Ii", "Var.Ii", "Z.Ii", Prname, Prname_rank, 
+        Prname_sim, "Skewness", "Kurtosis")
     if (!is.null(na.act) && excl) {
 	res <- naresid(na.act, res)
     }
@@ -194,9 +245,10 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     res
 }
 
+# "localmoran" quadr mean/median/pysal "Low-Low", "Low-High", "High-Low", "High-High"
 
 
-localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return_internals=TRUE, alternative = "two.sided", iseed=NULL) {
+localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, alternative = "two.sided", iseed=NULL, fix_i_in_Gstar_permutations=TRUE, no_repeat_in_row=FALSE) {
     if (!inherits(listw, "listw"))
 	stop(paste(deparse(substitute(listw)), "is not a listw object"))
     if (!is.numeric(x))
@@ -214,7 +266,9 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
     gstari <- FALSE
     if (!is.null(attr(listw$neighbours, "self.included")) &&
 	attr(listw$neighbours, "self.included")) gstari <- TRUE
+
     lx <- lag.listw(listw, x, zero.policy=zero.policy)
+
     x_star <- sum(x)
     if (gstari) {
         G <- lx/x_star
@@ -222,35 +276,76 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
         G <- lx/(x_star-c(x))
     }
 
-    cores <- get.coresOption()
-    if (is.null(cores)) {
-        parallel <- "no"
+# #124 repeat localG standard deviate direct output
+    if (gstari) {
+	xibar <- rep(mean(x), n)
+	si2 <- rep(sum(scale(x, scale=FALSE)^2)/n, n)
     } else {
-        parallel <- ifelse (get.mcOption(), "multicore", "snow")
+	xibar <- (rep(sum(x),n) - x) / (n - 1)
+	si2 <- ((rep(sum(x^2),n) - x^2) / (n - 1)) - xibar^2
     }
-    ncpus <- ifelse(is.null(cores), 1L, cores)
-    cl <- NULL
-    if (parallel == "snow") {
-        cl <- get.ClusterOption()
-        if (is.null(cl)) {
-            parallel <- "no"
-            warning("no cluster in ClusterOption, parallel set to no")
-        }
+    Wi <- sapply(listw$weights, sum)
+    S1i <- sapply(listw$weights, function(x) sum(x^2))
+    EG <- Wi*xibar
+    if (gstari) {
+        res <- G - (EG/x_star)
+    } else {
+        res <- G - (EG/(x_star-c(x)))
     }
-    if (!is.null(iseed)) {
-        stopifnot(is.numeric(iseed))
-        stopifnot(length(iseed) == 1L)
+    if (gstari) {
+        VG <- si2*((n*S1i - Wi^2)/(n-1))
+    } else {
+        VG <- si2*(((n-1)*S1i - Wi^2)/(n-2))
+    }
+    if (gstari) {
+        res <- res / sqrt(VG/x_star^2)
+    } else {
+        res <- res / sqrt(VG/(x_star-c(x))^2)
     }
 
-    permG_int <- function(i, xi, x_i, crdi, wtsi, nsim, Gi) {
+    crd <- card(listw$neighbours)
+    lww <- listw$weights
+
+    env <- new.env()
+    assign("x", x, envir=env)
+    assign("crd", crd, envir=env)
+    assign("lww", lww, envir=env)
+    assign("nsim", nsim, envir=env)
+    assign("G", G, envir=env)
+    assign("x_star", x_star, envir=env)
+    assign("gstari", gstari, envir=env)
+    assign("n", n, envir=env)
+    assign("no_repeat_in_row", no_repeat_in_row, envir=env)
+    varlist <- ls(envir = env)
+
+    permG_int <- function(i, env) {
         res_i <- rep(as.numeric(NA), 6)
-        if (crdi > 0) {
-            sx_i <- matrix(sample(x_i, size=crdi*nsim, replace=TRUE),
-                ncol=crdi, nrow=nsim)
-            lx_i <- sx_i %*% wtsi
-            res_p <- lx_i/(x_star-xi)
+        crdi <- get("crd", envir=env)[i]
+        n_i <- get("n", envir=env) - 1L
+        no_repeat_in_row <- get("no_repeat_in_row", envir=env)
+        if (crdi > 0) { # if i has neighbours
+            nsim <- get("nsim", envir=env)
+            xi <- get("x", envir=env)[i]
+            x_i <- get("x", envir=env)[-i]
+            if (no_repeat_in_row) {
+              samples <- .Call("perm_no_replace", as.integer(nsim),
+                as.integer(n_i), as.integer(crdi), PACKAGE="spdep")
+              sx_i <- matrix(x_i[samples], ncol=crdi, nrow=nsim)
+            } else {
+              sx_i <- matrix(sample(x_i, size=crdi*nsim, replace=TRUE),
+                ncol=crdi, nrow=nsim) # permute nsim*#neighbours from x[-i]
+            }
+            wtsi <- get("lww", envir=env)[[i]]
+            lx_i <- sx_i %*% wtsi # nsim by 1 = nsim by crdi %*% crdi by 1
+            # create nsim samples of Gi at i
+            x_star <- get("x_star", envir=env)
+            gstari <- get("gstari", envir=env)
+            # nsim by 1 = nsim by 1 / scalar
+            if (gstari) res_p <- lx_i/x_star
+            else res_p <- lx_i/(x_star-xi)
             res_i[1] <- mean(res_p)
             res_i[2] <- var(res_p)
+            Gi <- get("G", envir=env)[i]
 	    res_i[3] <- rank(c(res_p, Gi))[(nsim + 1L)]
             res_i[4] <- as.integer(sum(res_p >= Gi))
             res_i[5] <- e1071::skewness(res_p)
@@ -259,87 +354,81 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
         res_i
     }
 
-    crd <- card(listw$neighbours)
-    lww <- listw$weights
-
-    if (parallel == "snow") {
-      if (requireNamespace("parallel", quietly = TRUE)) {
-        sI <- parallel::splitIndices(n, length(cl))
-        env <- new.env()
-        assign("x", x, envir=env)
-        assign("crd", crd, envir=env)
-        assign("lww", lww, envir=env)
-        assign("nsim", nsim, envir=env)
-        assign("G", G, envir=env)
-        parallel::clusterExport(cl, varlist=c("x", "crd", "lww", "nsim", "G"),
-            envir=env)
-        if (!is.null(iseed)) parallel::clusterSetRNGStream(cl, iseed = iseed)
-        oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
- 	    permG_int(i, x[i], x[-i], crd[i], lww[[i]], nsim, G[i])})
-        out <- do.call("rbind", do.call("c", oo))
-        rm(env)
-      } else {
-        stop("parallel not available")
-      }
-    } else if (parallel == "multicore") {
-      if (requireNamespace("parallel", quietly = TRUE)) {
-        sI <- parallel::splitIndices(n, ncpus)
-        oldRNG <- RNGkind()
-        RNGkind("L'Ecuyer-CMRG")
-        if (!is.null(iseed)) set.seed(iseed)
-        oo <- parallel::mclapply(sI, FUN=lapply, function(i) {permG_int(i,
-            x[i], x[-i], crd[i], lww[[i]], nsim, G[i])}, mc.cores=ncpus)
-        RNGkind(oldRNG[1])
-        out <- do.call("rbind", do.call("c", oo))
-      } else {
-        stop("parallel not available")
-      }
-    } else {
-        if (!is.null(iseed)) set.seed(iseed)
-        oo <- lapply(1:n, function(i) permG_int(i, x[i], x[-i], 
-            crd[i], lww[[i]], nsim, G[i]))
-        out <- do.call("rbind", oo)
-    }
-    EG <- out[,1]
-    VG <- out[,2]
-
-    res <- (G - EG)
-    res <- res / sqrt(VG)
-    if (return_internals) {
-        gr <- punif((1:(nsim+1))/(nsim+1), 0, 1)
-        ls <- rev(gr)
-        ts <- (ifelse(gr > ls, ls, gr))*2
-        if (alternative == "two.sided") {
-            probs <- ts
-            Prname <- "Pr(z != E(Gi))"
-        } else if (alternative == "greater") {
-            Prname <- "Pr(z > E(Gi))"
-            probs <- gr
-        } else {
-            Prname <- "Pr(z < E(Gi))"
-            probs <- ls
+    permGstar_int <- function(i, env) {
+        res_i <- rep(as.numeric(NA), 6)
+        crdi <- get("crd", envir=env)[i]
+        n_i <- get("n", envir=env) - 1L
+        no_repeat_in_row <- get("no_repeat_in_row", envir=env)
+        if (crdi > 0) { # if i has neighbours
+            nsim <- get("nsim", envir=env)
+            xi <- get("x", envir=env)[i]
+            x_i <- get("x", envir=env)[-i]
+            if (no_repeat_in_row) {
+              samples <- .Call("perm_no_replace", as.integer(nsim),
+                as.integer(n_i), as.integer(crdi), PACKAGE="spdep")
+              sx_i <- matrix(x_i[samples], ncol=crdi, nrow=nsim)
+            } else {
+              sx_i <- matrix(sample(x_i, size=crdi*nsim, replace=TRUE),
+                ncol=crdi, nrow=nsim) # permute nsim*#neighbours from x[-i]
+            }
+            wtsi <- get("lww", envir=env)[[i]]
+            nbsi <- get("nbs", envir=env)[[i]]
+            ithnb <- which(nbsi == i)
+            sx_i[, ithnb] <- xi
+            lx_i <- sx_i %*% wtsi # nsim by 1 = nsim by crdi %*% crdi by 1
+            # create nsim samples of Gi at i
+            x_star <- get("x_star", envir=env)
+            # nsim by 1 = nsim by 1 / scalar
+            res_p <- lx_i/x_star
+            res_i[1] <- mean(res_p)
+            res_i[2] <- var(res_p)
+            Gi <- get("G", envir=env)[i]
+	    res_i[3] <- rank(c(res_p, Gi))[(nsim + 1L)]
+            res_i[4] <- as.integer(sum(res_p >= Gi))
+            res_i[5] <- e1071::skewness(res_p)
+            res_i[6] <- e1071::kurtosis(res_p)
         }
-        Prname_rank <- paste0(Prname, " Sim")
-        Prname_sim <- "Pr(folded) Sim"
-        if (alternative == "two.sided") 
-            pv <- 2 * pnorm(abs(res), lower.tail=FALSE)
-        else if (alternative == "greater") 
-            pv <- pnorm(res, lower.tail=FALSE)
-        else pv <- pnorm(res)
-        pu <- probs[as.integer(out[,3])]
-# 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
-        rnk0 <- as.integer(out[,4])
-        drnk0 <- nsim - rnk0
-        rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
-        pr <- (rnk + 1.0) / (nsim + 1.0)
-        resint <- cbind(G=G, EG=EG, VG=VG, pv=pv, pu=pu, pr=pr,
-            sk=out[,5], ku=out[,6])
-        colnames(resint) <- c("Gi", "E.Gi", "Var.Gi", Prname,
-            Prname_rank, Prname_sim, "Skewness", "Kurtosis")
-        attr(res, "internals") <- resint
+        res_i
     }
+
+    thisfun <- permG_int
+    if (gstari && fix_i_in_Gstar_permutations) {
+        thisfun <- permGstar_int
+        assign("nbs", listw$neighbours, envir=env)
+    }
+
+    out <- run_perm(fun=thisfun, idx=1:n, env=env, iseed=iseed, varlist=varlist)
+
+# add simulated standard deviate direct output
+    res_sim <- (G - out[,1])
+    res_sim <- res_sim / sqrt(out[,2])
+    probs <- probs_lut(stat="G", nsim=nsim, alternative=alternative)
+    Prname <- attr(probs, "Prname")
+    Prname_rank <- paste0(Prname, " Sim")
+    Prname_sim <- "Pr(folded) Sim"
+    if (alternative == "two.sided") 
+        pv <- 2 * pnorm(abs(res_sim), lower.tail=FALSE)
+    else if (alternative == "greater") 
+        pv <- pnorm(res_sim, lower.tail=FALSE)
+    else pv <- pnorm(res_sim)
+# look-up table
+    pu <- probs[as.integer(out[,3])]
+# 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
+    rnk0 <- as.integer(out[,4])
+    drnk0 <- nsim - rnk0
+    rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
+# folded
+    pr <- (rnk + 1.0) / (nsim + 1.0)
+    resint <- cbind(G=G, EG_sim=out[,1], VG_sim=out[,2], res_sim=res_sim,
+        pv=pv, pu=pu, pr=pr, sk=out[,5], ku=out[,6])
+    colnames(resint) <- c("Gi", "E.Gi", "Var.Gi", "StdDev.Gi", Prname,
+        Prname_rank, Prname_sim, "Skewness", "Kurtosis")
+    attr(res, "internals") <- resint
+    attr(res, "cluster") <- cut(x, c(-Inf, mean(x), Inf), labels = c("Low", "High"))
     attr(res, "gstari") <- gstari
     attr(res, "call") <- match.call()
     class(res) <- "localG"
     res
 }
+
+# "localG" cluster c("Low", "High")
