@@ -7,7 +7,7 @@ sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=NULL,
             zero.policy <- get("zeroPolicy", envir = .spdepOptions)
         stopifnot(is.logical(zero.policy))
         stopifnot(is.vector(var))
-	alternative <- match.arg(alternative, c("greater", "less"))
+	alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
 	if(!inherits(listw, "listw")) stop(paste(deparse(substitute(listw)),
 		"is not a listw object"))
 	if(!is.numeric(var)) stop(paste(deparse(substitute(var)),
@@ -27,7 +27,7 @@ sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=NULL,
 	
 	listw.U <- listw2U(listw)
 	mantel.moran <- function(x, listwU, zero.policy) {
-		res <- x * lag.listw(listw.U, x, zero.policy=zero.policy)
+		res <- x * lag.listw(listwU, x, zero.policy=zero.policy)
 		res <- sum(res)
 		res
 	}
@@ -53,18 +53,24 @@ sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=NULL,
                 var <- var[i]
                 return(f(x=var, ...))
             }
-            cl <- get("cl", envir = .spdepOptions)
-            if (!is.null(cl) && length(cl) > 1L) {
-                nnsim <- boot_wrapper_in(cl, nsim)
-                lres <- clusterCall(cl, boot, xs, statistic=mantel_boot,
-                    R=nnsim, sim="permutation", listwU=listw.U,
-                    zero.policy=zero.policy)
-                res <- boot_wrapper_out(lres, match.call())
+            cores <- get.coresOption()
+            if (is.null(cores)) {
+            parallel <- "no"
             } else {
-                res <- boot(xs, statistic=mantel_boot, R=nsim,
-                    sim="permutation", listwU=listw.U, 
-                    zero.policy=zero.policy)
+                parallel <- ifelse (get.mcOption(), "multicore", "snow")
             }
+            ncpus <- ifelse(is.null(cores), 1L, cores)
+            cl <- NULL
+            if (parallel == "snow") {
+                cl <- get.ClusterOption()
+                if (is.null(cl)) {
+                    parallel <- "no"
+                    warning("no cluster in ClusterOption, parallel set to no")
+                }
+            }
+            res <- boot(xs, statistic=mantel_boot, R=nsim,
+                sim="permutation", listwU=listw.U, 
+                zero.policy=zero.policy, parallel=parallel, ncpus=ncpus, cl=cl)
             return(res)
         }
 
@@ -82,6 +88,8 @@ sp.mantel.mc <- function(var, listw, nsim, type="moran", zero.policy=NULL,
         	pval <- punif((diff + 1)/(nsim + 1), lower.tail=FALSE)
     	else if (alternative == "greater") 
         	pval <- punif((diff + 1)/(nsim + 1))
+        else pval <- punif(abs(xrank - (nsim+1)/2)/(nsim + 1), 0, 0.5,
+                lower.tail=FALSE)
 	if (!is.finite(pval) || pval < 0 || pval > 1) 
 		warning("Out-of-range p-value: reconsider test arguments")
 	statistic <- res[nsim+1]
